/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2011  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82r                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_AT32UC3A.c 
Purpose : Initializes and handles the hardware for the OS as far
          as required by the OS for Atmel AT32UC3A0 CPU.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"
#include "RTOSVect.h"

/*********************************************************************
*
*       Configuration
*
*********************************************************************/

/*********************************************************************
*
*       Clock frequency settings
*/
#define   OS_FSYS       (48000000uL)

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ  (1000)
#endif

#ifndef   OS_PCLK_TIMER                /* Clock used for embOS timer */
  #define OS_PCLK_TIMER (OS_FSYS)
#endif

#ifndef   OS_PCLK_UART                 /* Peripheral clock for UART  */
  #define OS_PCLK_UART  (OS_FSYS)
#endif

/*********************************************************************
*
*       UART settings for OSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to OSView, please define it to be -1
*       Currently the standard code enables UART 0 per default
*/
#ifndef   OS_VIEW_ENABLE
  #define OS_VIEW_ENABLE  (1)
#endif

#ifndef   OS_UART
  #define OS_UART (0)
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400uL)
#endif

/*********************************************************************
*
*       UART baudrate calculation
*
*********************************************************************/

#define BAUDRATEREG ((OS_PCLK_UART + OS_BAUDRATE*8L) / (OS_BAUDRATE*16))
#if BAUDRATEREG > 0xFFFF
  #error Cannot set up UART for the given OS_FSYS and OS_BAUDRATE values.
#endif

/****** End of configuration settings *******************************/

#define OS_UART_USED (OS_VIEW_ENABLE && ((OS_UART == 0) || (OS_UART == 1)))

#define OS_TIMER_RELOAD (OS_PCLK_TIMER/OS_TICK_FREQ)

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
#define COUNT                  0x108
#define COMPARE                0x10c

#define MCCTRL                 (*(volatile unsigned int*) (0xFFFF0C00))
#define CKSEL                  (*(volatile unsigned int*) (0xFFFF0C04))
#define PLL0                   (*(volatile unsigned int*) (0xFFFF0C20))
#define OSCCTRL0               (*(volatile unsigned int*) (0xFFFF0C28))
#define ISR                    (*(volatile unsigned int*) (0xFFFF0C4C))
#define GCCTRL                 (*(volatile unsigned int*) (0xFFFF0C60))
#define FCR                    (*(volatile unsigned int*) (0xFFFE1400))
#define IRR0                   (*(volatile unsigned int*) (0xFFFF0900))

#if OS_UART_USED

#define USART0_BASE_ADDR       0xFFFF1400
#define USART1_BASE_ADDR       0xFFFF1800

#if   (OS_UART == 0)
  #define USART_BASE_ADDR      USART0_BASE_ADDR
  #define OS_UART_PIN_MASK     (0x3uL << 0)
  #define OS_UART_GROUP        5
  #define USART_FUNC_A         1
  #define USART_FUNC_B         0
  #define USART_FUNC_C         0
#elif (OS_UART == 1)
  #define USART_BASE_ADDR      USART1_BASE_ADDR
  #define OS_UART_PIN_MASK     (0x3uL << 5)
  #define OS_UART_GROUP        6
  #define USART_FUNC_A         1
  #define USART_FUNC_B         0
  #define USART_FUNC_C         0
#endif

#define USART_CR               (*(volatile unsigned int*) (USART_BASE_ADDR + 0x00))
#define USART_MR               (*(volatile unsigned int*) (USART_BASE_ADDR + 0x04))
#define USART_IER              (*(volatile unsigned int*) (USART_BASE_ADDR + 0x08))
#define USART_IDR              (*(volatile unsigned int*) (USART_BASE_ADDR + 0x0C))
#define USART_IMR              (*(volatile unsigned int*) (USART_BASE_ADDR + 0x10))
#define USART_CSR              (*(volatile unsigned int*) (USART_BASE_ADDR + 0x14))
#define USART_RHR              (*(volatile unsigned int*) (USART_BASE_ADDR + 0x18))
#define USART_THR              (*(volatile unsigned int*) (USART_BASE_ADDR + 0x1C))
#define USART_BRGR             (*(volatile unsigned int*) (USART_BASE_ADDR + 0x20))
#define USART_RTOR             (*(volatile unsigned int*) (USART_BASE_ADDR + 0x24))
#define USART_TTGR             (*(volatile unsigned int*) (USART_BASE_ADDR + 0x28))
#define TXE_BIT                1

#define PORT0_BASE_ADDR        0xFFFF1000
#define PORT0_GPERS            (*(volatile unsigned int*) (PORT0_BASE_ADDR + 0x04))
#define PORT0_GPERC            (*(volatile unsigned int*) (PORT0_BASE_ADDR + 0x08))
#define PORT0_PMR0S            (*(volatile unsigned int*) (PORT0_BASE_ADDR + 0x14))
#define PORT0_PMR0C            (*(volatile unsigned int*) (PORT0_BASE_ADDR + 0x18))
#define PORT0_PMR1S            (*(volatile unsigned int*) (PORT0_BASE_ADDR + 0x24))
#define PORT0_PMR1C            (*(volatile unsigned int*) (PORT0_BASE_ADDR + 0x28))
#define PORT0_ODERC            (*(volatile unsigned int*) (PORT0_BASE_ADDR + 0x48))

#endif  /* OS_UART_USED */

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_Systick()
*
*       The embOS timer tick handler function, called from _OS_ISR_Tick()
*/
void OS_Systick(void) {
  OS_SetCompare(OS_TIMER_RELOAD);
  OS_TICK_Handle();
}

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  OS_IncDI();
  //
  // Initialize PLL
  //
  OSCCTRL0    = 0x00000707uL;           // Mode 1 and 560 usec startup time
  MCCTRL      = 0x00000004uL;           // Enable Oscillator 0
  while ((ISR & 0x00000080uL) == 0);    // Wait until clock ready
  MCCTRL      = 0x00000005uL;           // Select Oscillator 0 as source for the main clock
  PLL0        = 0x100F020CuL;           // Mul = 16, div=2, pllcount=16, OPT = 3 => fvco = 96MHz with 80MHz <fvco <180MHz, fPLL = fvco /2
  PLL0        = 0x100F020DuL;           // Enable PLL0
  while ((ISR & 0x00000001uL) == 0);    // Wait until ready to be used as clock source
  GCCTRL      = 0x00000002uL;           // Select PLL 0 as source for the generic clock, no division
  GCCTRL      = 0x00000006uL;           // Enable Generic Clock 0
  CKSEL       = 0x00000000uL;           // PBADIV = 0: PBA clock equals main clock
  while ((ISR & 0x00000020uL) == 0);    // Wait until ready to be used as clock source
  FCR         = 0x00000040uL;           // Set 1 wait state for Flash
  MCCTRL      = 0x00000006uL;           // Select PLL0 as source for the main clock
  //
  // Initialize OS timer
  //
  OS_SetCompare(OS_TIMER_RELOAD);
  OS_SetCount(0);
  //
  // Initialize interrupt group registers
  //
  OS_ISRInit();
  //
  // Set priority 0 for group 0 (count/compare timer)
  //
  OS_ISRSetGroupPrio(0, 0);      
    
  OS_COM_Init();
  OS_DecRI();
}

/*********************************************************************
*
*       Idle loop  OS_Idle()
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggling an output or incrementing a counter)
*/
void OS_Idle(void) {     // Idle loop: No task is ready to exec
  while (1) {
#if DEBUG == 0
// Instruction to put CPU in halt mode to save power
#endif
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for high
*       resolution time maesurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  int    Count;

  Time  = OS_Time;
  Count = OS_GetSVCCount();
  if (IRR0 & 0x01) {  // Counter interrupt pending, adjust time
    Time++;
    Count = OS_GetSVCCount();
  }
  return (OS_TIMER_RELOAD) * Time + Count;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       Communication for OSView via UART (optional)
*
**********************************************************************
*/

#if OS_UART_USED

/*********************************************************************
*
*       USART_IRQHandler()
*
*       Used for embOSView, called from OS_COM_ISR_Usart().
*/
void USART_IRQHandler(void) {
  char c;
  if ((USART_CSR & (1uL << 0)) != 0) {        // Character received?
    c = USART_RHR;                            // Process character, ignore errors
    OS_OnRx(c);
  }
  if (((USART_CSR & (1uL << TXE_BIT)) != 0) && ((USART_IMR & (1uL << TXE_BIT)) != 0)) {
    USART_IDR = (1uL << TXE_BIT);             // Disable Tx interrupt
    OS_OnTx();                                // Send next, if any...
  }
  USART_CR = (1uL << 8);                      // Reset any errors
}

/*********************************************************************
*
*       OS_COM_Send1()
*       Never call this function directly from your application
*/
void OS_COM_Send1(OS_U8 c) {
  USART_THR = c;                  // Send character
  USART_IER = (1uL << TXE_BIT);   // Enable interrupt for TxRDY / TxEmpty
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART for embOSView
*/
void OS_COM_Init(void) {
  PORT0_ODERC = OS_UART_PIN_MASK;     // Disable output driver of UART pins
  PORT0_GPERS = OS_UART_PIN_MASK;     // Temporarily set UART pins to PIO mode
  PORT0_PMR0C = OS_UART_PIN_MASK;     // Clear Configuration bit0
  PORT0_PMR1C = OS_UART_PIN_MASK;     // Clear Configuration bit1, select Function A
  PORT0_PMR0S = (OS_UART_PIN_MASK & USART_FUNC_B);
  PORT0_PMR1S = (OS_UART_PIN_MASK & USART_FUNC_C);
  PORT0_GPERC = OS_UART_PIN_MASK;     // Enable special function on UART pins

  USART_IDR   = 0xFFFFFFFFuL;         // Disable all uart interrupts
  USART_MR    = 0x00000000uL;         // Reset mode register
  USART_RTOR  = 0x00000000uL;         // Reset receive timeout register
  USART_TTGR  = 0x00000000uL;         // Reset transmitter timeguard register
  USART_CR    = 0x000A610CuL;         // Reset usart, disable dtr and rts
  USART_BRGR  = BAUDRATEREG;          // Set baudrate to selected baud rate
  USART_MR    = 0x000008C0;           // Set mode: Oversampling mode 16, Normal, MCK, 8 data bits, async mode, NO parity, 1 stop bit
  USART_IER   = 0x00000001;           // Interrupt enabled for: RXRDY
  USART_CR   |= (1uL << 8);           // Reset any Errors
  USART_CR   |= 0x00000050;           // Enable rx and tx
  //
  // Set priority 0 for int5/6 group (USART0/USART1)
  //
  OS_ISRSetGroupPrio(OS_UART_GROUP, 0);
}

#else  /* selected UART not supported, using dummies */

void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid compiler warning */
  OS_COM_ClearTxActive();  /* Let the OS know that Tx is not busy */
}

#endif /*  OS_UART_USED  */

/****** Final check of configuration ********************************/

#ifndef OS_UART_USED
  #error "OS_UART_USED has to be defined"
#endif

/*********************************************************************
*
*       OS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_ISRInit()
*
*       Initializes all interrupt group registers
*/
void OS_ISRInit(void) {
  OS_U32 *p;
  OS_U8 i;
  //
  // Set all group registers to default values 
  // which are interrupt priority 0 and _int0 handler
  //
  p = (OS_U32*)AVR32_INTC_IPR_BASE_ADDR;
  for (i = 0; i < AVR32_INTC_NUM_INT_GRPS; i++) {
    p[i] = (OS_U32)ipr_val[0];
  }
}

/*********************************************************************
*
*       OS_ISRHandler
*
*       Handles all embOS interrupts, the handler checks the 
*       interrupt cause and interrupt request register to calculate
*       the pending interrupt and calls the corresponding interrupt
*       service routine
*/
void OS_ISRHandler(OS_U32 Prio) {
  OS_ISR_HANDLER *pISR;
  OS_U32 ISRGroup;
  OS_U32 ISRRequest;
  OS_U16 ISRIndex;
  OS_U8  n, i;
  //
  // Get group number from interrupt cause register (ICR)
  //
  ISRGroup = *((OS_U32*)(AVR32_INTC_ICR_BASE_ADDR + ((AVR32_PRIO_MAX - Prio) * 0x04)));
  //
  // Get interrupt group request register (IRR)
  //
  ISRRequest = *((OS_U32*)(AVR32_INTC_IRR_BASE_ADDR + (ISRGroup * 0x04)));
  //
  // Check if any group is pending
  //
  if (ISRRequest != 0) {
    //
    // Get highest pending interrupt in group
    //
    n = vector_group_entries[ISRGroup];
    while (((ISRRequest & (1ul << (n - 1))) == 0x00) && (n > 0)) {
      n--;
    }
    // 
    // Check if an interrupt was pending
    // 
    if (n > 0) {
      //
      // Calculate position in vector table
      //
      ISRIndex = 0;
      for (i = 0; i < ISRGroup; i++) {
        ISRIndex += vector_group_entries[i];
      }
      ISRIndex += (n - 1);
      //
      // Get ISR handler and call it
      //
      pISR = vector_table[ISRIndex];
      pISR();
    }
  }
}

/*********************************************************************
*
*       OS_ISRSetGroupPrio
*
*       Set an interrupt priority for a group. 
*       The group number is device specific.
*       The priority is a value between 0 and 3.
*/
void OS_ISRSetGroupPrio(OS_U32 ISRGroup, OS_U32 Prio) {
  OS_U32 *p;
#if DEBUG
  if (ISRGroup >= AVR32_INTC_LAST_INT_GRPS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
  if (Prio > AVR32_PRIO_MAX) {
    OS_Error(OS_ERR_ISR_PRIO);
  }
#endif
  //
  // Set group register
  //
  p = (OS_U32*)AVR32_INTC_IPR_BASE_ADDR;
  p[ISRGroup] = (Prio << AVR32_INTLEVEL_OFFSET) | (OS_U32)ipr_val[Prio];
}

/********************* End of File **********************************/
