/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2011  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82r                                            *
*                                                                    *
**********************************************************************

--------------------------------------------------------------------
File        : RTOSVect.c 
Purpose     : Interrupt functions for Atmel AT32UC3A0 CPU
--------------- END-OF-HEADER ----------------------------------------
*/

#include "RTOS.h"
#include "RTOSVect.h"
#include "OS_Config.h"

#ifndef   OS_VIEW_ENABLE
  #define OS_VIEW_ENABLE  (1)
#endif

#ifndef   OS_UART
  #define OS_UART (0)
#endif

#define OS_UART_USED (OS_VIEW_ENABLE && ((OS_UART == 0) || (OS_UART == 1)))

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _IllegalException()
*
*       Is called from any exception handler which is not implemented
*       by the application (not overwritten by user).
*       We implemented and endless loop, so the programm stops when any
*       of the exceptions is called.
*       Using a debugger and setting a breakpoint here allows to analyze
*       which exception / interrupt was called by examinig the call stack.
*       Normally, the CPU should not arrive here.
*/
static void _IllegalException(void) {
  while(1);
}

/*********************************************************************
*
*       Default dummy interrupt handler
*
*       We implemented a call to _IllegalException(), so the program
*       calls this function when any of the exceptions is called.
*       Using a debugger and setting a breakpoint in _IllegelException()
*       allows to analyze the call stack and shows
*       which exception / interrupt was called.
*/
void COMPARE_IRQHandler      (void) { _IllegalException(); }
void EIM0_IRQHandler         (void) { _IllegalException(); }
void EIM1_IRQHandler         (void) { _IllegalException(); }
void EIM2_IRQHandler         (void) { _IllegalException(); }
void EIM3_IRQHandler         (void) { _IllegalException(); }
void EIM4_IRQHandler         (void) { _IllegalException(); }
void EIM5_IRQHandler         (void) { _IllegalException(); }
void EIM6_IRQHandler         (void) { _IllegalException(); }
void EIM7_IRQHandler         (void) { _IllegalException(); }
void RTC_IRQHandler          (void) { _IllegalException(); }
void PM_IRQHandler           (void) { _IllegalException(); }
void FREQM_IRQHandler        (void) { _IllegalException(); }
void GPIO0_IRQHandler        (void) { _IllegalException(); }
void GPIO1_IRQHandler        (void) { _IllegalException(); }
void GPIO2_IRQHandler        (void) { _IllegalException(); }
void GPIO3_IRQHandler        (void) { _IllegalException(); }
void GPIO4_IRQHandler        (void) { _IllegalException(); }
void GPIO5_IRQHandler        (void) { _IllegalException(); }
void GPIO6_IRQHandler        (void) { _IllegalException(); }
void GPIO7_IRQHandler        (void) { _IllegalException(); }
void GPIO8_IRQHandler        (void) { _IllegalException(); }
void GPIO9_IRQHandler        (void) { _IllegalException(); }
void GPIO10_IRQHandler       (void) { _IllegalException(); }
void GPIO11_IRQHandler       (void) { _IllegalException(); }
void GPIO12_IRQHandler       (void) { _IllegalException(); }
void GPIO13_IRQHandler       (void) { _IllegalException(); }
void PDMA0_IRQHandler        (void) { _IllegalException(); }
void PDMA1_IRQHandler        (void) { _IllegalException(); }
void PDMA2_IRQHandler        (void) { _IllegalException(); }
void PDMA3_IRQHandler        (void) { _IllegalException(); }
void PDMA4_IRQHandler        (void) { _IllegalException(); }
void PDMA5_IRQHandler        (void) { _IllegalException(); }
void PDMA6_IRQHandler        (void) { _IllegalException(); }
void PDMA7_IRQHandler        (void) { _IllegalException(); }
void PDMA8_IRQHandler        (void) { _IllegalException(); }
void PDMA9_IRQHandler        (void) { _IllegalException(); }
void PDMA10_IRQHandler       (void) { _IllegalException(); }
void PDMA11_IRQHandler       (void) { _IllegalException(); }
void PDMA12_IRQHandler       (void) { _IllegalException(); }
void PDMA13_IRQHandler       (void) { _IllegalException(); }
void PDMA14_IRQHandler       (void) { _IllegalException(); }
void HFLASHC_IRQHandler      (void) { _IllegalException(); }
void USART0_IRQHandler       (void) { _IllegalException(); }
void USART1_IRQHandler       (void) { _IllegalException(); }
void USART2_IRQHandler       (void) { _IllegalException(); }
void USART3_IRQHandler       (void) { _IllegalException(); }
void SPI0_IRQHandler         (void) { _IllegalException(); }
void SPI1_IRQHandler         (void) { _IllegalException(); }
void TWI_IRQHandler          (void) { _IllegalException(); }
void PWM_IRQHandler          (void) { _IllegalException(); }
void SSC_IRQHandler          (void) { _IllegalException(); }
void TC0_IRQHandler          (void) { _IllegalException(); }
void TC1_IRQHandler          (void) { _IllegalException(); }
void TC2_IRQHandler          (void) { _IllegalException(); }
void ADC_IRQHandler          (void) { _IllegalException(); }
void MAC_IRQHandler          (void) { _IllegalException(); }
void USB_IRQHandler          (void) { _IllegalException(); }
void HSDRAMC_IRQHandler      (void) { _IllegalException(); }
void DAC_IRQHandler          (void) { _IllegalException(); }

void OS_Systick(void);
#if OS_UART_USED
  void USART_IRQHandler(void);
#endif

/*********************************************************************
*
*       Exception vector table
*/
const OS_ISR_HANDLER*  vector_table[] = {
  //(const OS_ISR_HANDLER*) COMPARE_IRQHandler,        // Group 0  Line 0  SYSBLOCK  COMPARE
  (const OS_ISR_HANDLER*) OS_Systick,
  (const OS_ISR_HANDLER*) EIM0_IRQHandler,           // Group 1  Line 0  External Interrupt Controller EIM 0
  (const OS_ISR_HANDLER*) EIM1_IRQHandler,           // Group 1  Line 1  External Interrupt Controller EIM 1
  (const OS_ISR_HANDLER*) EIM2_IRQHandler,           // Group 1  Line 2  External Interrupt Controller EIM 2
  (const OS_ISR_HANDLER*) EIM3_IRQHandler,           // Group 1  Line 3  External Interrupt Controller EIM 3
  (const OS_ISR_HANDLER*) EIM4_IRQHandler,           // Group 1  Line 4  External Interrupt Controller EIM 4
  (const OS_ISR_HANDLER*) EIM5_IRQHandler,           // Group 1  Line 5  External Interrupt Controller EIM 5
  (const OS_ISR_HANDLER*) EIM6_IRQHandler,           // Group 1  Line 6  External Interrupt Controller EIM 6
  (const OS_ISR_HANDLER*) EIM7_IRQHandler,           // Group 1  Line 7  External Interrupt Controller EIM 7
  (const OS_ISR_HANDLER*) RTC_IRQHandler,            // Group 1  Line 8  Real Time Counter RTC
  (const OS_ISR_HANDLER*) PM_IRQHandler,             // Group 1  Line 9  Power Manager PM
  (const OS_ISR_HANDLER*) FREQM_IRQHandler,          // Group 1  Line 10 Frequency Meter FREQM
  (const OS_ISR_HANDLER*) GPIO0_IRQHandler,          // Group 2  Line 0  General Purpose Input/Output Controller GPIO 0
  (const OS_ISR_HANDLER*) GPIO1_IRQHandler,          // Group 2  Line 1  General Purpose Input/Output Controller GPIO 1
  (const OS_ISR_HANDLER*) GPIO2_IRQHandler,          // Group 2  Line 2  General Purpose Input/Output Controller GPIO 2
  (const OS_ISR_HANDLER*) GPIO3_IRQHandler,          // Group 2  Line 3  General Purpose Input/Output Controller GPIO 3
  (const OS_ISR_HANDLER*) GPIO4_IRQHandler,          // Group 2  Line 4  General Purpose Input/Output Controller GPIO 4
  (const OS_ISR_HANDLER*) GPIO5_IRQHandler,          // Group 2  Line 5  General Purpose Input/Output Controller GPIO 5
  (const OS_ISR_HANDLER*) GPIO6_IRQHandler,          // Group 2  Line 6  General Purpose Input/Output Controller GPIO 6
  (const OS_ISR_HANDLER*) GPIO7_IRQHandler,          // Group 2  Line 7  General Purpose Input/Output Controller GPIO 7
  (const OS_ISR_HANDLER*) GPIO8_IRQHandler,          // Group 2  Line 8  General Purpose Input/Output Controller GPIO 8
  (const OS_ISR_HANDLER*) GPIO9_IRQHandler,          // Group 2  Line 9  General Purpose Input/Output Controller GPIO 9
  (const OS_ISR_HANDLER*) GPIO10_IRQHandler,         // Group 2  Line 10 General Purpose Input/Output Controller GPIO 10
  (const OS_ISR_HANDLER*) GPIO11_IRQHandler,         // Group 2  Line 11 General Purpose Input/Output Controller GPIO 11
  (const OS_ISR_HANDLER*) GPIO12_IRQHandler,         // Group 2  Line 12 General Purpose Input/Output Controller GPIO 12
  (const OS_ISR_HANDLER*) GPIO13_IRQHandler,         // Group 2  Line 13 General Purpose Input/Output Controller GPIO 13
  (const OS_ISR_HANDLER*) PDMA0_IRQHandler,          // Group 3  Line 0  Peripheral DMA Controller PDMA 0
  (const OS_ISR_HANDLER*) PDMA1_IRQHandler,          // Group 3  Line 1  Peripheral DMA Controller PDMA 1
  (const OS_ISR_HANDLER*) PDMA2_IRQHandler,          // Group 3  Line 2  Peripheral DMA Controller PDMA 2
  (const OS_ISR_HANDLER*) PDMA3_IRQHandler,          // Group 3  Line 3  Peripheral DMA Controller PDMA 3
  (const OS_ISR_HANDLER*) PDMA4_IRQHandler,          // Group 3  Line 4  Peripheral DMA Controller PDMA 4
  (const OS_ISR_HANDLER*) PDMA5_IRQHandler,          // Group 3  Line 5  Peripheral DMA Controller PDMA 5
  (const OS_ISR_HANDLER*) PDMA6_IRQHandler,          // Group 3  Line 6  Peripheral DMA Controller PDMA 6
  (const OS_ISR_HANDLER*) PDMA7_IRQHandler,          // Group 3  Line 7  Peripheral DMA Controller PDMA 7
  (const OS_ISR_HANDLER*) PDMA8_IRQHandler,          // Group 3  Line 8  Peripheral DMA Controller PDMA 8
  (const OS_ISR_HANDLER*) PDMA9_IRQHandler,          // Group 3  Line 9  Peripheral DMA Controller PDMA 9
  (const OS_ISR_HANDLER*) PDMA10_IRQHandler,         // Group 3  Line 10 Peripheral DMA Controller PDMA 10
  (const OS_ISR_HANDLER*) PDMA11_IRQHandler,         // Group 3  Line 11 Peripheral DMA Controller PDMA 11
  (const OS_ISR_HANDLER*) PDMA12_IRQHandler,         // Group 3  Line 12 Peripheral DMA Controller PDMA 12
  (const OS_ISR_HANDLER*) PDMA13_IRQHandler,         // Group 3  Line 13 Peripheral DMA Controller PDMA 13
  (const OS_ISR_HANDLER*) PDMA14_IRQHandler,         // Group 3  Line 14 Peripheral DMA Controller PDMA 14
  (const OS_ISR_HANDLER*) HFLASHC_IRQHandler,        // Group 4  Line 0  Flash Controller HFLASHC
#if OS_UART_USED
  (const OS_ISR_HANDLER*) USART_IRQHandler,
#else
  (const OS_ISR_HANDLER*) USART0_IRQHandler,         // Group 5  Line 0  Universal Synchronous/Asynchronous Receiver/Transmitter USART0
#endif
  (const OS_ISR_HANDLER*) USART1_IRQHandler,         // Group 6  Line 0  Universal Synchronous/Asynchronous Receiver/Transmitter USART1
  (const OS_ISR_HANDLER*) USART2_IRQHandler,         // Group 7  Line 0  Universal Synchronous/Asynchronous Receiver/Transmitter USART2
  (const OS_ISR_HANDLER*) USART3_IRQHandler,         // Group 8  Line 0  Universal Synchronous/Asynchronous Receiver/Transmitter USART3
  (const OS_ISR_HANDLER*) SPI0_IRQHandler,           // Group 9  Line 0  Serial Peripheral Interface SPI0
  (const OS_ISR_HANDLER*) SPI1_IRQHandler,           // Group 10 Line 0  Serial Peripheral Interface SPI1
  (const OS_ISR_HANDLER*) TWI_IRQHandler,            // Group 11 Line 0  Two-wire Interface TWI
  (const OS_ISR_HANDLER*) PWM_IRQHandler,            // Group 12 Line 0  Pulse Width Modulation Controller PWM
  (const OS_ISR_HANDLER*) SSC_IRQHandler,            // Group 13 Line 0  Synchronous Serial Controller SSC
  (const OS_ISR_HANDLER*) TC0_IRQHandler,            // Group 14 Line 0  Timer/Counter TC0
  (const OS_ISR_HANDLER*) TC1_IRQHandler,            // Group 14 Line 1  Timer/Counter TC1
  (const OS_ISR_HANDLER*) TC2_IRQHandler,            // Group 14 Line 2  Timer/Counter TC2
  (const OS_ISR_HANDLER*) ADC_IRQHandler,            // Group 15 Line 0  Analog to Digital Converter ADC
  (const OS_ISR_HANDLER*) MAC_IRQHandler,            // Group 16 Line 0  Ethernet MAC MACB
  (const OS_ISR_HANDLER*) USB_IRQHandler,            // Group 17 Line 0  USB 2.0 OTG Interface USB
  (const OS_ISR_HANDLER*) HSDRAMC_IRQHandler,        // Group 18 Line 0  SDRAM Controller HSDRAMC
  (const OS_ISR_HANDLER*) DAC_IRQHandler             // Group 19 Line 0  Audio Bitstream DAC DAC
};

/********************* End of File **********************************/
