/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2011  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82r                                            *
*                                                                    *
**********************************************************************

--------------------------------------------------------------------
File        : RTOSVect.h
Purpose     : Interrupt function defines for Atmel AT32UC3A0 CPU
--------------- END-OF-HEADER ----------------------------------------
*/

#ifndef RTOSVECT_H                           /* avoid multiple inclusion */
#define RTOSVECT_H

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define AVR32_INTC_BASE_ADDR               0xFFFF0800
#define AVR32_INTC_IPR_BASE_ADDR           (AVR32_INTC_BASE_ADDR + 0x00)
#define AVR32_IPR0                         (*(volatile OS_U32*) (AVR32_INTC_IPR_BASE_ADDR + 0x00))
#define AVR32_IPR1                         (*(volatile OS_U32*) (AVR32_INTC_IPR_BASE_ADDR + 0x08))
#define AVR32_IPR2                         (*(volatile OS_U32*) (AVR32_INTC_IPR_BASE_ADDR + 0x08))
#define AVR32_IPR3                         (*(volatile OS_U32*) (AVR32_INTC_IPR_BASE_ADDR + 0x0C))
#define AVR32_INTC_ICR_BASE_ADDR           (AVR32_INTC_BASE_ADDR + 0x200)
#define AVR32_INTC_IRR_BASE_ADDR           (AVR32_INTC_BASE_ADDR + 0x100)

#define AVR32_PRIO_MAX                     3
#define AVR32_INTLEVEL_OFFSET              30

#define AVR32_INTC_NUM_INT_GRPS            20
#define AVR32_INTC_LAST_INT_GRPS           20
#define AVR32_INTC_NUM_IRQS_PER_GRP0       1
#define AVR32_INTC_NUM_IRQS_PER_GRP1       11
#define AVR32_INTC_NUM_IRQS_PER_GRP2       14
#define AVR32_INTC_NUM_IRQS_PER_GRP3       15
#define AVR32_INTC_NUM_IRQS_PER_GRP4       1
#define AVR32_INTC_NUM_IRQS_PER_GRP5       1
#define AVR32_INTC_NUM_IRQS_PER_GRP6       1
#define AVR32_INTC_NUM_IRQS_PER_GRP7       1
#define AVR32_INTC_NUM_IRQS_PER_GRP8       1
#define AVR32_INTC_NUM_IRQS_PER_GRP9       1
#define AVR32_INTC_NUM_IRQS_PER_GRP10      1
#define AVR32_INTC_NUM_IRQS_PER_GRP11      1
#define AVR32_INTC_NUM_IRQS_PER_GRP12      1
#define AVR32_INTC_NUM_IRQS_PER_GRP13      1
#define AVR32_INTC_NUM_IRQS_PER_GRP14      3
#define AVR32_INTC_NUM_IRQS_PER_GRP15      1
#define AVR32_INTC_NUM_IRQS_PER_GRP16      1
#define AVR32_INTC_NUM_IRQS_PER_GRP17      1
#define AVR32_INTC_NUM_IRQS_PER_GRP18      1
#define AVR32_INTC_NUM_IRQS_PER_GRP19      1

/*********************************************************************
*
*       External data
*
**********************************************************************
*/
extern const unsigned int ipr_val[AVR32_PRIO_MAX];
extern const OS_ISR_HANDLER* vector_table[];

/*********************************************************************
*
*       Const data
*
**********************************************************************
*/
static const char vector_group_entries[] = {
  AVR32_INTC_NUM_IRQS_PER_GRP0,
  AVR32_INTC_NUM_IRQS_PER_GRP1,
  AVR32_INTC_NUM_IRQS_PER_GRP2,
  AVR32_INTC_NUM_IRQS_PER_GRP3,
  AVR32_INTC_NUM_IRQS_PER_GRP4,
  AVR32_INTC_NUM_IRQS_PER_GRP5,
  AVR32_INTC_NUM_IRQS_PER_GRP6,
  AVR32_INTC_NUM_IRQS_PER_GRP7,
  AVR32_INTC_NUM_IRQS_PER_GRP8,
  AVR32_INTC_NUM_IRQS_PER_GRP9,
  AVR32_INTC_NUM_IRQS_PER_GRP10,
  AVR32_INTC_NUM_IRQS_PER_GRP11,
  AVR32_INTC_NUM_IRQS_PER_GRP12,
  AVR32_INTC_NUM_IRQS_PER_GRP13,
  AVR32_INTC_NUM_IRQS_PER_GRP14,
  AVR32_INTC_NUM_IRQS_PER_GRP15,
  AVR32_INTC_NUM_IRQS_PER_GRP16,
  AVR32_INTC_NUM_IRQS_PER_GRP17,
  AVR32_INTC_NUM_IRQS_PER_GRP18,
  AVR32_INTC_NUM_IRQS_PER_GRP19,
};

#endif                                  /* avoid multiple inclusion */

/********************* End of File **********************************/
