/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2011  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: Internal                                         *
*                                                                    *
*       Current version number will be inserted here                 *
*       when shipment is built.                                      *
*                                                                    *
**********************************************************************

--------------------------------------------------------------------
File        : exception.s
Purpose     : Assembler exception functions
--------------- END-OF-HEADER ----------------------------------------
*/

/*********************************************************************
*
*       Publics
*/
         .global _evba

/*********************************************************************
*
*       Exception Vector Table
*
* EVBA must be aligned with a power of two strictly greater than the EVBA-
* relative offset of the last vector.
*/
         .section  .exception, "ax", @progbits
  
         .balign 0x200
         .type _evba, @function
_evba:

        .org  0x000
_handle_Unrecoverable_Exception:      // Unrecoverable Exception.
        rjmp $

        .org  0x004
_handle_TLB_Multiple_Hit:             // TLB Multiple Hit: UNUSED IN AVR32UC.
        rjmp $

        .org  0x008                   
_handle_Bus_Error_Data_Fetch:         // Bus Error Data Fetch
        rjmp $

        .org  0x00C               
_handle_Bus_Error_Instruction_Fetch:  // Bus Error Instruction Fetch.
        rjmp $

        .org  0x010                   
_handle_NMI:                          // NMI
        rjmp $

        .org  0x014                   
_handle_Instruction_Address:          // Instruction Address
        rjmp $

        .org  0x018                   
_handle_ITLB_Protection:              // ITLB Protection
        rjmp $

        .org  0x01C                   
_handle_Breakpoint:                   // Breakpoint
        rjmp $

        .org  0x020                   
_handle_Illegal_Opcode:               // Illegal Opcode
        rjmp $

        .org  0x024                   
_handle_Unimplemented_Instruction:    // Unimplemented Instruction
        rjmp $

        .org  0x028
_handle_Privilege_Violation:          // Privilege Violation.
        rjmp $

        .org  0x02C
_handle_Floating_Point:               // Floating-Point: UNUSED IN AVR32UC.
        rjmp $

        .org  0x030
_handle_Coprocessor_Absent:           // Coprocessor Absent: UNUSED IN AVR32UC.
        rjmp $

        .org  0x034
_handle_Data_Address_Read:            // Data Address (Read)
        rjmp $

        .org  0x038
_handle_Data_Address_Write:           // Data Address (Write). 
        rjmp $

        .org  0x03C
_handle_DTLB_Protection_Read:         // DTLB Protection (Read).
        rjmp $

        .org  0x040
_handle_DTLB_Protection_Write:        // DTLB Protection (Write)
        rjmp $

        .org  0x044
_handle_DTLB_Modified:                // DTLB Modified: UNUSED IN AVR32UC
        rjmp $

        .org  0x050
_handle_ITLB_Miss:                    // ITLB Miss: UNUSED IN AVR32UC
        rjmp $

        .org  0x060
_handle_DTLB_Miss_Read:               // DTLB Miss (Read): UNUSED IN AVR32UC
        rjmp $

        .org  0x070
_handle_DTLB_Miss_Write:              // DTLB Miss (Write): UNUSED IN AVR32UC
        rjmp $

        .org  0x100                   
_handle_Supervisor_Call:              // Supervisor Call
        rjmp scall_handler


//
// Low level interrupt handler for all four interrupt levels
// Please replace OS_CallISR with OS_CallNestableISR if you want to use nestable interrupts for a priority
//
  .balign 4

_int0:
        mov     r12, 0
        call    OS_CallISR
        rete

_int1:
        mov     r12, 1
        call    OS_CallISR
        rete

_int2:
        mov     r12, 2
        call    OS_CallISR
        rete

_int3:
        mov     r12, 3
        call    OS_CallISR
        rete

  .balign 4

//
// Values to store in the interrupt priority registers for the various interrupt priority levels
//
        .global ipr_val
        .type ipr_val, @object
ipr_val:
        .word (_int0 - _evba), (_int1 - _evba), (_int2 - _evba), (_int3 - _evba)


/****** End Of File *************************************************/
