/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2011  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82r                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for AVR32 EVK1101
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/
#define AVR32_GPIO_PIOA_BASE_ADDR    (0xFFFF1000)

#define PIO_GPER_OFFS                (0x00) // GPIO Enable Register
#define PIO_ODR_OFFS                 (0x40) // Output Driver Enable Register 
#define PIO_OVR_OFFS                 (0x50) // Output Value Register
#define PIO_OVRS_OFFS                (0x54) // Output Value Register Set
#define PIO_OVRC_OFFS                (0x58) // Output Value Register Clear
#define PIO_OVRT_OFFS                (0x5C) // Output Value Register Toogle

/****** arget specific configuration *******************************/
#define LEDPORT_BASE_ADDR            (AVR32_GPIO_PIOA_BASE_ADDR)
#define LED0_BIT                     (7)    // LED0 on EVK1101 board, active low 
#define LED1_BIT                     (8)    // LED1 on EVK1101 board, active low 
#define LED2_BIT                     (21)   // LED2 on EVK1101 board, active low 
#define LED3_BIT                     (22)   // LED3 on EVK1101 board, active low 

#define LEDPORT_GPER                 (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_GPER_OFFS))
#define LEDPORT_ODR                  (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_ODR_OFFS))
#define LEDPORT_OVRS                 (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_OVRS_OFFS))
#define LEDPORT_OVRC                 (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_OVRC_OFFS))
#define LEDPORT_OVRT                 (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_OVRT_OFFS))
#define LEDPORT_STATE                (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_OVR_OFFS))

#define LEDMASK_ALL                  ((1uL << LED0_BIT) | (1uL << LED1_BIT) | (1uL << LED2_BIT) | (1uL << LED3_BIT))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  LEDPORT_GPER  = LEDMASK_ALL;  // Enable Port           
  LEDPORT_ODR   = LEDMASK_ALL;  // Switch to output mode 
  LEDPORT_OVRS |= LEDMASK_ALL;  // Initially clear LEDs  
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void BSP_SetLED(int Index) {
  if (Index < 4) {
    LEDPORT_OVRC = (1uL << (Index + LED0_BIT));       // Switch on LED  
  }
}

void BSP_ClrLED(int Index) {
  if (Index < 4) {
    LEDPORT_OVRS = (1uL << (Index + LED0_BIT));         // Switch off LED 
  }
}

void BSP_ToggleLED(int Index) {
  if (Index < 4) {
    LEDPORT_OVRT = (1uL << (Index + LED0_BIT));       // Toogle LED     
  }
}

/********************* End of File **********************************/
