/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2011  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82r                                            *
*                                                                    *
**********************************************************************

--------------------------------------------------------------------
File        : RTOSVect.c 
Purpose     : Interrupt functions for Atmel AT32UC3A3 CPU
--------------- END-OF-HEADER ----------------------------------------
*/

#include "RTOS.h"
#include "RTOSVect.h"

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _IllegalException()
*
*       Is called from any exception handler which is not implemented
*       by the application (not overwritten by user).
*       We implemented and endless loop, so the programm stops when any
*       of the exceptions is called.
*       Using a debugger and setting a breakpoint here allows to analyze
*       which exception / interrupt was called by examinig the call stack.
*       Normally, the CPU should not arrive here.
*/
static void _IllegalException(void) {
  while(1);
}

/*********************************************************************
*
*       Default dummy interrupt handler
*
*       We implemented a call to _IllegalException(), so the program
*       calls this function when any of the exceptions is called.
*       Using a debugger and setting a breakpoint in _IllegelException()
*       allows to analyze the call stack and shows
*       which exception / interrupt was called.
*/
void COMPARE_IRQHandler      (void) { _IllegalException(); }
void EIM0_IRQHandler         (void) { _IllegalException(); }
void EIM1_IRQHandler         (void) { _IllegalException(); }                                
void EIM2_IRQHandler         (void) { _IllegalException(); }
void EIM3_IRQHandler         (void) { _IllegalException(); }
void EIM4_IRQHandler         (void) { _IllegalException(); }
void EIM5_IRQHandler         (void) { _IllegalException(); } 
void EIM6_IRQHandler         (void) { _IllegalException(); }
void EIM7_IRQHandler         (void) { _IllegalException(); }
void PM_IRQHandler           (void) { _IllegalException(); }
void GPIO0_IRQHandler        (void) { _IllegalException(); } 
void GPIO1_IRQHandler        (void) { _IllegalException(); }
void GPIO2_IRQHandler        (void) { _IllegalException(); }
void GPIO3_IRQHandler        (void) { _IllegalException(); }
void GPIO4_IRQHandler        (void) { _IllegalException(); } 
void GPIO5_IRQHandler        (void) { _IllegalException(); }
void GPIO6_IRQHandler        (void) { _IllegalException(); }
void GPIO7_IRQHandler        (void) { _IllegalException(); }
void GPIO8_IRQHandler        (void) { _IllegalException(); } 
void GPIO9_IRQHandler        (void) { _IllegalException(); }
void GPIO10_IRQHandler       (void) { _IllegalException(); }
void GPIO11_IRQHandler       (void) { _IllegalException(); }
void GPIO12_IRQHandler       (void) { _IllegalException(); } 
void GPIO13_IRQHandler       (void) { _IllegalException(); }
void GPIO14_IRQHandler       (void) { _IllegalException(); }
void PDCA0_IRQHandler        (void) { _IllegalException(); }
void PDCA1_IRQHandler        (void) { _IllegalException(); } 
void PDCA2_IRQHandler        (void) { _IllegalException(); }
void PDCA3_IRQHandler        (void) { _IllegalException(); }
void PDCA4_IRQHandler        (void) { _IllegalException(); }
void PDCA5_IRQHandler        (void) { _IllegalException(); } 
void PDCA6_IRQHandler        (void) { _IllegalException(); }
void PDCA7_IRQHandler        (void) { _IllegalException(); }
void FLASHC_IRQHandler       (void) { _IllegalException(); }
void USART0_IRQHandler       (void) { _IllegalException(); } 
void USART1_IRQHandler       (void) { _IllegalException(); }
void USART2_IRQHandler       (void) { _IllegalException(); }
void USART3_IRQHandler       (void) { _IllegalException(); }
void SPI0_IRQHandler         (void) { _IllegalException(); } 
void SPI1_IRQHandler         (void) { _IllegalException(); }
void TWIM0_IRQHandler        (void) { _IllegalException(); }
void TWIM1_IRQHandler        (void) { _IllegalException(); }
void SSC_IRQHandler          (void) { _IllegalException(); } 
void TC0_IRQHandler          (void) { _IllegalException(); }
void TC1_IRQHandler          (void) { _IllegalException(); }
void TC2_IRQHandler          (void) { _IllegalException(); }
void ADC_IRQHandler          (void) { _IllegalException(); } 
void TC10_IRQHandler         (void) { _IllegalException(); }
void TC11_IRQHandler         (void) { _IllegalException(); }
void TC12_IRQHandler         (void) { _IllegalException(); }
void USB_IRQHandler          (void) { _IllegalException(); }
void SDRAMC_IRQHandler       (void) { _IllegalException(); }
void DAC_IRQHandler          (void) { _IllegalException(); }
void MCI_IRQHandler          (void) { _IllegalException(); } 
void AES_IRQHandler          (void) { _IllegalException(); }
void DMACA_BLOCK_IRQHandler  (void) { _IllegalException(); }
void DMACA_DSTT_IRQHandler   (void) { _IllegalException(); }
void DMACA_ERR_IRQHandler    (void) { _IllegalException(); }
void DMACA_SRCT_IRQHandler   (void) { _IllegalException(); }
void DMACA_TFR_IRQHandler    (void) { _IllegalException(); }
void MSI_IRQHandler          (void) { _IllegalException(); }
void TWIS0_IRQHandler        (void) { _IllegalException(); }
void TWIS1_IRQHandler        (void) { _IllegalException(); }
void RS4_IRQHandler          (void) { _IllegalException(); }

void OS_Systick(void);
void USART_IRQHandler(void);
void TC_IRQHandler (void);

/*********************************************************************
*
*       Exception vector table
*/
const OS_ISR_HANDLER*  vector_table[] = {
  //(const OS_ISR_HANDLER*) COMPARE_IRQHandler,        // Group 0  Line 0  SYSBLOCK  COMPARE
  (const OS_ISR_HANDLER*) OS_Systick,
  (const OS_ISR_HANDLER*) EIM0_IRQHandler,           // Group 1  Line 0  External Interrupt Controller EIM 0
  (const OS_ISR_HANDLER*) EIM1_IRQHandler,           // Group 1  Line 1  External Interrupt Controller EIM 1
  (const OS_ISR_HANDLER*) EIM2_IRQHandler,           // Group 1  Line 2  External Interrupt Controller EIM 2
  (const OS_ISR_HANDLER*) EIM3_IRQHandler,           // Group 1  Line 3  External Interrupt Controller EIM 3
  (const OS_ISR_HANDLER*) EIM4_IRQHandler,           // Group 1  Line 4  External Interrupt Controller EIM 4
  (const OS_ISR_HANDLER*) EIM5_IRQHandler,           // Group 1  Line 5  External Interrupt Controller EIM 5
  (const OS_ISR_HANDLER*) EIM6_IRQHandler,           // Group 1  Line 6  External Interrupt Controller EIM 6
  (const OS_ISR_HANDLER*) EIM7_IRQHandler,           // Group 1  Line 7  External Interrupt Controller EIM 7
  (const OS_ISR_HANDLER*) PM_IRQHandler,             // Group 1  Line 9  Power Manager PM
  (const OS_ISR_HANDLER*) GPIO0_IRQHandler,          // Group 2  Line 0  General Purpose Input/Output Controller GPIO 0
  (const OS_ISR_HANDLER*) GPIO1_IRQHandler,          // Group 2  Line 1  General Purpose Input/Output Controller GPIO 1
  (const OS_ISR_HANDLER*) GPIO2_IRQHandler,          // Group 2  Line 2  General Purpose Input/Output Controller GPIO 2
  (const OS_ISR_HANDLER*) GPIO3_IRQHandler,          // Group 2  Line 3  General Purpose Input/Output Controller GPIO 3
  (const OS_ISR_HANDLER*) GPIO4_IRQHandler,          // Group 2  Line 4  General Purpose Input/Output Controller GPIO 4
  (const OS_ISR_HANDLER*) GPIO5_IRQHandler,          // Group 2  Line 5  General Purpose Input/Output Controller GPIO 5
  (const OS_ISR_HANDLER*) GPIO6_IRQHandler,          // Group 2  Line 6  General Purpose Input/Output Controller GPIO 6
  (const OS_ISR_HANDLER*) GPIO7_IRQHandler,          // Group 2  Line 7  General Purpose Input/Output Controller GPIO 7
  (const OS_ISR_HANDLER*) GPIO8_IRQHandler,          // Group 2  Line 8  General Purpose Input/Output Controller GPIO 8
  (const OS_ISR_HANDLER*) GPIO9_IRQHandler,          // Group 2  Line 9  General Purpose Input/Output Controller GPIO 9
  (const OS_ISR_HANDLER*) GPIO10_IRQHandler,         // Group 2  Line 10 General Purpose Input/Output Controller GPIO 10
  (const OS_ISR_HANDLER*) GPIO11_IRQHandler,         // Group 2  Line 11 General Purpose Input/Output Controller GPIO 11
  (const OS_ISR_HANDLER*) GPIO12_IRQHandler,         // Group 2  Line 12 General Purpose Input/Output Controller GPIO 12
  (const OS_ISR_HANDLER*) GPIO13_IRQHandler,         // Group 2  Line 13 General Purpose Input/Output Controller GPIO 13
  (const OS_ISR_HANDLER*) GPIO14_IRQHandler,         // Group 2  Line 14 General Purpose Input/Output Controller GPIO 14
  (const OS_ISR_HANDLER*) PDCA0_IRQHandler,          // Group 3  Line 0  Peripheral Direct Memory Access PDCA 0
  (const OS_ISR_HANDLER*) PDCA1_IRQHandler,          // Group 3  Line 1  Peripheral Direct Memory Access PDCA 1
  (const OS_ISR_HANDLER*) PDCA2_IRQHandler,          // Group 3  Line 2  Peripheral Direct Memory Access PDCA 2
  (const OS_ISR_HANDLER*) PDCA3_IRQHandler,          // Group 3  Line 3  Peripheral Direct Memory Access PDCA 3
  (const OS_ISR_HANDLER*) PDCA4_IRQHandler,          // Group 3  Line 4  Peripheral Direct Memory Access PDCA 4
  (const OS_ISR_HANDLER*) PDCA5_IRQHandler,          // Group 3  Line 5  Peripheral Direct Memory Access PDCA 5
  (const OS_ISR_HANDLER*) PDCA6_IRQHandler,          // Group 3  Line 6  Peripheral Direct Memory Access PDCA 6
  (const OS_ISR_HANDLER*) PDCA7_IRQHandler,          // Group 3  Line 7  Peripheral Direct Memory Access PDCA 7
  (const OS_ISR_HANDLER*) FLASHC_IRQHandler,         // Group 4  Line 0  Flash Controller FLASHC
  (const OS_ISR_HANDLER*) USART0_IRQHandler,         // Group 5  Line 0  Universal Synchronous/Asynchronous Receiver/Transmitter USART0
  (const OS_ISR_HANDLER*) USART1_IRQHandler,         // Group 6  Line 0  Universal Synchronous/Asynchronous Receiver/Transmitter USART1
  (const OS_ISR_HANDLER*) USART2_IRQHandler,         // Group 7  Line 0  Universal Synchronous/Asynchronous Receiver/Transmitter USART2
  (const OS_ISR_HANDLER*) USART3_IRQHandler,         // Group 8  Line 0  Universal Synchronous/Asynchronous Receiver/Transmitter USART3
  (const OS_ISR_HANDLER*) SPI0_IRQHandler,           // Group 9  Line 0  Serial Peripheral Interface SPI0
  (const OS_ISR_HANDLER*) SPI1_IRQHandler,           // Group 10 Line 0  Serial Peripheral Interface SPI1
  (const OS_ISR_HANDLER*) TWIM0_IRQHandler,          // Group 11 Line 0  Two-wire Interface TWI
  (const OS_ISR_HANDLER*) TWIM1_IRQHandler,          // Group 12 Line 0  Two-wire Interface TWI
  (const OS_ISR_HANDLER*) SSC_IRQHandler,            // Group 13 Line 0  Synchronous Serial Controller SSC
  (const OS_ISR_HANDLER*) TC0_IRQHandler,            // Group 14 Line 0  Timer/Counter TC0
  (const OS_ISR_HANDLER*) TC1_IRQHandler,            // Group 14 Line 1  Timer/Counter TC1
  (const OS_ISR_HANDLER*) TC2_IRQHandler,            // Group 14 Line 2  Timer/Counter TC2
  (const OS_ISR_HANDLER*) ADC_IRQHandler,            // Group 15 Line 0  Analog to Digital Converter ADC
  (const OS_ISR_HANDLER*) TC10_IRQHandler,           // Group 16 Line 0  Timer/Counter TC10
  (const OS_ISR_HANDLER*) TC11_IRQHandler,           // Group 16 Line 1  Timer/Counter TC11
  (const OS_ISR_HANDLER*) TC12_IRQHandler,           // Group 16 Line 2  Timer/Counter TC12
  (const OS_ISR_HANDLER*) USB_IRQHandler,            // Group 17 Line 0  USB 2.0 OTG Interface USB
  (const OS_ISR_HANDLER*) SDRAMC_IRQHandler,         // Group 18 Line 0  SDRAM Controller
  (const OS_ISR_HANDLER*) DAC_IRQHandler,            // Group 19 Line 0  Audio Bitstream DAC
  (const OS_ISR_HANDLER*) MCI_IRQHandler,            // Group 20 Line 0  Multimedia Card Interface MCI
  (const OS_ISR_HANDLER*) AES_IRQHandler,            // Group 21 Line 0  Advanced Encryption Standard
  (const OS_ISR_HANDLER*) DMACA_BLOCK_IRQHandler,    // Group 22 Line 0  DMA Controller DMACA BLOCK
  (const OS_ISR_HANDLER*) DMACA_DSTT_IRQHandler,     // Group 22 Line 1  DMA Controller DMACA DSTT
  (const OS_ISR_HANDLER*) DMACA_ERR_IRQHandler,      // Group 22 Line 2  DMA Controller DMACA ERR
  (const OS_ISR_HANDLER*) DMACA_SRCT_IRQHandler,     // Group 22 Line 3  DMA Controller DMACA SRCT
  (const OS_ISR_HANDLER*) DMACA_TFR_IRQHandler,      // Group 22 Line 4  DMA Controller DMACA TFR
  (const OS_ISR_HANDLER*) MSI_IRQHandler,            // Group 26 Line 0  Memory Stick Interface MSI
  (const OS_ISR_HANDLER*) TWIS0_IRQHandler,          // Group 27 Line 0  Two wire slave intercae TWIS0
  (const OS_ISR_HANDLER*) TWIS1_IRQHandler,          // Group 28 Line 0  Two wire slave intercae TWIS1
  (const OS_ISR_HANDLER*) RS4_IRQHandler,            // Group 29 Line 0  Error code corrector Hamming and Reed Solomon RS4
};

/********************* End of File **********************************/
